;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xdfb20f9390c027f3ec055aa0d8d0450f . "./code.cg2js/cg-acache.fasl")
(#x2434554c6e0ebd6f184c55b16d143c2b . "./code.cg2js/cg-base.fasl")
(#x3474f31be68c4f7a9f93d0d64d399bff . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xe08b3236603287cf0bda4c95277ac4f6 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x8fb624d626cd314833c21dd980031ce3 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x6c615aec04b5e07d538935b208e8c585 . "./code.cg2js/cg-button.fasl")
(#x944fb149b043ae154e303fb6a0c55154 . "./code.cg2js/cg-calendar.fasl")
(#x7795b034c97f470ce1808ed0263639bb . "./code.cg2js/cg-caret.fasl")
(#xf2632279acd420fae50084edb98beedf . "./code.cg2js/cg-chart-or-plot.fasl")
(#x60a8e0e9f0962185da6519ea36751061 . "./code.cg2js/cg-chart-widget.fasl")
(#xb2c88916ce583d76f2eeac33cbee315b . "./code.cg2js/cg-check-box.fasl")
(#xa6c07ca72727e43c643c9c6de202978f . "./code.cg2js/cg-choice-list.fasl")
(#x8bf405fbd65a1a823af0759073fdfb3e . "./code.cg2js/cg-choose-printer.fasl")
(#x2c921a9f4bc2261af02cc82dae9d0afd . "./code.cg2js/cg-class-grid.fasl")
(#x8128097337580076a9ef66ac1656ffb5 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8a31112d483231c77d1c29f95f4b82cc . "./code.cg2js/cg-class-support.fasl")
(#x843d247b103a62f086b43ed4232ad38f . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xe0457343de1fb54e6741ca2176fe8c2e . "./code.cg2js/cg-clipboard-stack.fasl")
(#xdcfbd7ff4b38b30ba6e69d7a9f4d7ee7 . "./code.cg2js/cg-clipboard.fasl")
(#x3d453cfe9f404840a40b7d2d7805c0c0 . "./code.cg2js/cg-color-dialog.fasl")
(#xf738f8c6a18f4ff58796a9aae9863e2f . "./code.cg2js/cg-combo-box.fasl")
(#xcac2b0ae212c3fabe68cac0571b246c0 . "./code.cg2js/cg-common-control.fasl")
(#x6ef7d44aa21e9469660d63dd6ec4ac4a . "./code.cg2js/cg-comtab.fasl")
(#x29cb1636ce71d3dc0b7354ff9cd13814 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x6376f6c1a8c27226d6a7e33b10fb27a4 . "./code.cg2js/cg-curve.fasl")
(#x8b7e0ffcf9e320977dd336f13372c4b1 . "./code.cg2js/cg-dde-utils.fasl")
(#x89e908a0870d0d5ef3d404df7dd85bb8 . "./code.cg2js/cg-dev.fasl")
(#xe8a1a35d190aedaf75a385162ee3d422 . "./code.cg2js/cg-dialog-item.fasl")
(#x75d2261f875e3bd58f8502f9503482aa . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xfe23224938610bdbea092a6088e1d47f . "./code.cg2js/cg-directory-dialog.fasl")
(#xd70c1dfffa39d39d1ffd13970ca6dd50 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xe0bb62fa28e8dbdad90cfd3e56620c88 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x565626a30f7eb8ac7b58c340dcd02e5a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x5ee4f14d4e50d2f5500ffd47ccf649c9 . "./code.cg2js/cg-drawable.fasl")
(#xfa1190c536d2c671055a3beff5e09fbf . "./code.cg2js/cg-dropping-outline.fasl")
(#xfa20008290ea055998b6435c09b40a25 . "./code.cg2js/cg-edit-in-place.fasl")
(#x04ecc862a14d86ef323d4f06580034df . "./code.cg2js/cg-editable-text.fasl")
(#xc424a3b199cd93dcc93876c87f668f0f . "./code.cg2js/cg-file-dialog.fasl")
(#xeacc6fe9aacdd4550b786c2bb718f8b3 . "./code.cg2js/cg-fill-texture.fasl")
(#x54807435c2e3d8f89978839694de2de3 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x5c43b5f2d583a9909a71515dce639f4f . "./code.cg2js/cg-font-dialog.fasl")
(#x5296d749fc7021d6b544ceb783eeb480 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc88250d3714389220cfe22b7a89df073 . "./code.cg2js/cg-get-pixmap.fasl")
(#x09f4e0370438e73c8ee665c0147e1a25 . "./code.cg2js/cg-get-position.fasl")
(#x909a51cdf583d494ce7e7d8cd057dbed . "./code.cg2js/cg-graphics-context.fasl")
(#x9570af658e04de452f7fcf8fb4b86f63 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x26d8f072f78b6e0d16240f657c2082dd . "./code.cg2js/cg-grid-widget.fasl")
(#x71cf939bcf5129f39a12cdbe0c82d669 . "./code.cg2js/cg-group-box.fasl")
(#x102ca9befccbd48e95a23dc0e29181ba . "./code.cg2js/cg-header-control.fasl")
(#xa998c206d4dfd0d3291d26fefbe8eb19 . "./code.cg2js/cg-hotspot.fasl")
(#xf8ee4d0490fd71823cc0ed9d2541044a . "./code.cg2js/cg-html-dialog.fasl")
(#x87671a03d9c42e961f38360353c2416a . "./code.cg2js/cg-html-widget.fasl")
(#xbd451394f1247cc51b871faab6b17bf5 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xf88545bf11652e7a644f244e4fa68bde . "./code.cg2js/cg-icon.fasl")
(#x760523643686c7e9fb21409e4d5653f2 . "./code.cg2js/cg-ie.fasl")
(#xb6da602025fa11f9a106d8b5f46d019e . "./code.cg2js/cg-intersect-posbox.fasl")
(#xb9005b9e0a6918eb2dbfdcc99ddaddfe . "./code.cg2js/cg-intersect.fasl")
(#x95627afe99183df3cb842fa454ea4705 . "./code.cg2js/cg-item-list.fasl")
(#x363ff440c989d60ab41d376e6f31b53f . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x383860410df0d1e9741d961ae14b1f1d . "./code.cg2js/cg-lamp.fasl")
(#xb7c0c8c61c7e51ec9b10e79ae751465b . "./code.cg2js/cg-layout.fasl")
(#x5cde488839e3ac3bdfda8c32b16db8ce . "./code.cg2js/cg-lettered-menu.fasl")
(#x9d0c555c2cb6c71c42faa10600a7d8d4 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x2080fcaf9b3c8a7bb7f0692207b5f543 . "./code.cg2js/cg-lisp-text.fasl")
(#x83742f8744a693865ad20a48c5fd6715 . "./code.cg2js/cg-lisp-widget.fasl")
(#x6b287525b258d739e95ea1e4eb3f00e2 . "./code.cg2js/cg-list-view.fasl")
(#x04f339efeaab91793dd1ea8fe9a71944 . "./code.cg2js/cg-mci.fasl")
(#x41352c40fd30b9b0c04e69b538b49731 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xec80703d8e19db29a49dd85a28c72a82 . "./code.cg2js/cg-menu.fasl")
(#x27ba2bf76e5e1c62fda568fe345d5463 . "./code.cg2js/cg-message-dialog.fasl")
(#x129596b528f1960273fed8082369b904 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xae528a92baeedafb6010bbaa41ca1ee7 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x748f4a8401b786f76d0ab30e29923ba6 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x1a14cbb6c3847ebb6aa2f9702d1fcdcb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa072b98adb09b567c5dc697a3ede2599 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa5038fdc95cfd8d67fe54ec5fe03f03c . "./code.cg2js/cg-nodes.fasl")
(#xf16945b3a2020b157d66189c2e594c26 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x52df3bf644f129515567d121894fce22 . "./code.cg2js/cg-object-editor.fasl")
(#x2cdf7e6f31554e708534e167880ab60b . "./code.cg2js/cg-ocx.fasl")
(#xce8093d57b4b6670dca7ef12096e4900 . "./code.cg2js/cg-os-widget.fasl")
(#xa4ad0f39c6a59ad156f749951111e0a0 . "./code.cg2js/cg-os-window.fasl")
(#x46ddde1e434c2d0daccd6227a96f72d8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x49a886160ff56269fa88376df416bfa9 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xaa9d3fbf0eb1bd915d9d4dfb72afe8d8 . "./code.cg2js/cg-outline.fasl")
(#x2a749bfbaccfbed5dd2a06849bffc1fa . "./code.cg2js/cg-palette.fasl")
(#x165af9e44d3e0f4bae64ca4b39e91960 . "./code.cg2js/cg-paren-matching.fasl")
(#xc8e709faac6b0259d875f98b32fad492 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc829e843c4755dbe0dceac95537c4f90 . "./code.cg2js/cg-picture-widget.fasl")
(#x66c9221a8caab45fd842321cb01b726f . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x878cfd279550cbea415db5f2a3446685 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x1900e41d543ccca94613b5afbff33f10 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbfc65982ad5dcc755d48f9326baf3b1e . "./code.cg2js/cg-pixmap-widget.fasl")
(#x2055690600eed3df4d422c02397e94ac . "./code.cg2js/cg-pixmap.fasl")
(#xc67852954c157cd53e0c055ef24bd14f . "./code.cg2js/cg-plot-widget.fasl")
(#x22c6dd77ad183bf740fff8a42cf0b389 . "./code.cg2js/cg-printing.fasl")
(#x4309708c81c7264fbd8cea0977f104af . "./code.cg2js/cg-progress-indicator.fasl")
(#xb18a309057dc16676dcecdef2b84e945 . "./code.cg2js/cg-project-window.fasl")
(#x1885987b834bc8e714a7a66dde604b0e . "./code.cg2js/cg-property.fasl")
(#xc504b8a89232289096b7931cd6e11db2 . "./code.cg2js/cg-radio-button.fasl")
(#x0cbb77c96f5e2c14919d8c8942158646 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x0c529935c6d1b8b92d6d99dda9667b20 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x68e15045970ece0093bd36aa3582572c . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xffd19c79fa48f5143e356a40e2bd3aff . "./code.cg2js/cg-rich-edit.fasl")
(#x4f16b812f635d14d891636d01dea8733 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xacd61517e0912c5cfa6299070899abe9 . "./code.cg2js/cg-scaling-stream.fasl")
(#xe37c99be7c77d7c156e565e27fca5dcb . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xee5309a0a7a16913e18d5e7182ee0fd4 . "./code.cg2js/cg-scroll-bar.fasl")
(#xd445cb5f4163b32565ecdaba30d13441 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd9475b2968da29ab01b5d5b59a38c90a . "./code.cg2js/cg-selected-object.fasl")
(#x63ba70cdccb7f22478632aa9b32fb283 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb194894f483289759111987a41577978 . "./code.cg2js/cg-split-bar.fasl")
(#x555b434ed96da5889768db085f021976 . "./code.cg2js/cg-static-text.fasl")
(#x51de325a042c36de69be76f6dd58170c . "./code.cg2js/cg-status-bar.fasl")
(#x82452b7820bd5515886068ae458fed75 . "./code.cg2js/cg-string-dialog.fasl")
(#x35ef29ff82bf7a14be7ec0ed5eb01fcc . "./code.cg2js/cg-tab-control.fasl")
(#x55e14af0b4998e50794be7b6cecdd3f3 . "./code.cg2js/cg-template-string.fasl")
(#xf1bcc2e4eda6828967de713ed82cad1b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x61eb0fca5f3181b43e732a892dc778fa . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x5657be792250b9e88e8d0dc72702606a . "./code.cg2js/cg-text-edit-pane.fasl")
(#xa92923f1851015f248106ba54945b322 . "./code.cg2js/cg-text-or-combo.fasl")
(#xd66f0a66557d159e8329173d4eeac4ef . "./code.cg2js/cg-text-widget.fasl")
(#x01817e33189b8131a2f23215004c6ad2 . "./code.cg2js/cg-timer.fasl")
(#x941072407df148235e6ac9464c212038 . "./code.cg2js/cg-toggling-widget.fasl")
(#xf4f8c18d103165c63536efd7aa19b0b3 . "./code.cg2js/cg-toolbar.fasl")
(#x58c0d0f586d941b16e6884aa3ba72867 . "./code.cg2js/cg-tooltip.fasl")
(#xc82c9e5d081e98384af877984c3e091d . "./code.cg2js/cg-touch.fasl")
(#xd87a1a2053078a842ff6ccffa904cf20 . "./code.cg2js/cg-trackbar.fasl")
(#xfa3680e992a944cc2efff6c24134a940 . "./code.cg2js/cg-tray.fasl")
(#xe2aced8f30cf2fd607ea6dbe8760bfac . "./code.cg2js/cg-up-down-control.fasl")
(#x1f294bb31641de632ae1bba7de645ce9 . "./code.cg2js/cg-utility-dialog.fasl")
(#x0e4c5fe15c3f86d1f5e77dada90a8d1a . "./code.cg2js/cg-web-browser-dde.fasl")
(#xeeb8bb2393d8e40f97e30389afe20ca2 . "./code.cg2js/cg-web-browser.fasl")
(#x80d5ca37c5fcff8f87d22b55783e7048 . "./code.cg2js/cg-wrap-string.fasl")
(#xaf2a463cf346c03d246f1c9c5ca60fd6 . "./code.cg2js/cg-yes-no-list.fasl")
(#xddf46766a984a779cd8dc8d49988e431 . "./code.cg2js/cg-yes-no-string.fasl")
(#x5e3f1dc7e083bfc82eaa27a2ea0fdec3 . "./code.cg2js/cg.fasl")
(#xc5e735d6b9acac8700f88a9a86350e58 . "./code.idejs/ide-acache.fasl")
(#x455da3c4ceb2c17dbc9a69d1fdcced01 . "./code.idejs/ide-apropos.fasl")
(#xb82be35d186c51ab400b9ceff19df8f7 . "./code.idejs/ide-base.fasl")
(#x935361a0fbe5bbd7d987965541b8cfce . "./code.idejs/ide-class-browser.fasl")
(#x005ec7b3607d9cacc30a0536b65120b4 . "./code.idejs/ide-class-grapher.fasl")
(#xdb54def68595742194f477c3dc067cb1 . "./code.idejs/ide-clipboard.fasl")
(#xd5728f6399b3b386244ea18d60494c0f . "./code.idejs/ide-dev.fasl")
(#x03263a0599afd1bd137e0f4bb935af8a . "./code.idejs/ide-editor.fasl")
(#xc06c87c179b5e31a9b5dc282945166c4 . "./code.idejs/ide-find-definitions.fasl")
(#xfa8eb4b21800d761e18127ae47d866e2 . "./code.idejs/ide-find-in-files.fasl")
(#x8214178be514368db6b52a34d05b2b66 . "./code.idejs/ide-form.fasl")
(#x716465841433e197bf3d4e65a7ec0e63 . "./code.idejs/ide-grapher.fasl")
(#x8bcb9e037346c31d6b707b4d25a86ab0 . "./code.idejs/ide-inspector.fasl")
(#x391560b3ba94ca1430f46a18a29de2e6 . "./code.idejs/ide-intro.fasl")
(#xb6d485696cd3875ac5d32444e2425c2f . "./code.idejs/ide-keyshort.fasl")
(#x609bd16bd0d1f50bd5b26234334ed039 . "./code.idejs/ide-navigator.fasl")
(#x7a92034083fb1204ea9de2b89bdebc89 . "./code.idejs/ide-package-list.fasl")
(#x4072c6091b60ddfc3fa3f9e3e06c3506 . "./code.idejs/ide-patch-dialog.fasl")
(#x3f6bfabddbb66ef5079546a9434ea4f6 . "./code.idejs/ide-process-browser.fasl")
(#x75571b1c80f295d3a87f5ebc3d0e237e . "./code.idejs/ide-project.fasl")
(#x9fead9d077612642d7a3b7359502dcc7 . "./code.idejs/ide-trace.fasl")
(#x2c3c60f819d486917a5b87fed168c34b . "./code.idejs/ide-window-list.fasl")
(#x9fbaa092923728f8e774ccf9fe085fd4 . "./code.idejs/ide.fasl")
(#xc2912bce53894841a1e9ff8f0fd00d64 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
